# -*- coding: utf-8 -*-

"""Generate a default configuration-file section for {{package}}"""

from __future__ import print_function


def config_section_data():
    """Produce the default configuration section for app.config,
       when called by `resilient-circuits config [-c|-u]`
    """
#    config_data = u"""[{{package}}]
#setting=xxx
#"""
#    return config_data
    return None
