#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

""" nosetests --with-coverage --cover-package=resilient_circuits """


def test():
    """The most basic tests"""
    from resilient_circuits import actions_component
    from resilient_circuits import app
    from resilient_circuits import component_loader
    from resilient_circuits import keyring_arguments
    from resilient_circuits import rest_helper
    from resilient_circuits import template_functions
    pass
